#lang racket

(require web-server/managers/none)
(require web-server/servlet/web)
(require web-server/http)
(require web-server/templates)
(require web-server/dispatch)
(require web-server/private/mime-types)
(require net/url)

(require "model.rkt")

(provide interface-version manager start)

(define interface-version 'v2)

(define manager
  (create-none-manager
   (lambda (req)
     `(html (head (title "No Continuations Here!"))
            (body (h1 "No Continuations Here!"))))))

;; ('a -> (U #f 'a)) * request * symbol -> (U #f 'a)
;; Get parameters from a request, with a given conversion func
(define (param/conv conv req name)
  (let ([possible-binding 
         (bindings-assq
          (string->bytes/utf-8 (symbol->string name))
          (request-bindings/raw req))])
    (match possible-binding
      [(? binding:form? b) (conv (bytes->string/utf-8
                                  (binding:form-value b)))]
      [_ #f])))

(define (param-number req name) (param/conv string->number req name))
(define (param-string req name) (param/conv (λ(x)x) req name))
(define (param-symbol req name) (param/conv string->symbol req name))

 
(define (respond/fail code message)
  (send/back
   (response/full
    code (string->bytes/utf-8 message)
    (current-seconds) TEXT/HTML-MIME-TYPE
    empty
    (list #"<p>Error</p>"
          (string->bytes/utf-8 message)))))

(define (main-page req auth?)
  (send/suspend/dispatch
   (λ (embed/url)
        (let* ([main-style (include-template "static/main.css")]
               [add-link-url (embed/url handle-add-link)]
               [all-links-url (embed/url handle-all-links)]
               [home-url (embed/url (λ (req) (main-page req auth?)))]
               [login-fragment (if auth? "You are logged in."
                                   (let ([login-url (embed/url handle-login)])
                                     (include-template "static/login.html")))]
               [add-link-fragment (if auth? (include-template "static/add-link.html")
                                      (include-template "static/add-link-no-login.html"))]
               [main-js (include-template "static/main.js")]
               [ad (pick-ad)]
               [links (take (all-links) (min (length (all-links)) 10))])
          (list #"text/html" (include-template "static/main.html"))))))

(define (start req)
  (main-page req #f))

(define (handle-login req)
  (let* ([uname (param-string req 'uname)]
         [passwd (param-string req 'passwd)]
         [stored-passwd (hash-query users 'uname uname)])
    (if (and stored-passwd
             (equal? (hash-ref stored-passwd 'passwd) passwd))
        (begin (redirect/get)
               (main-page req #t))
        (begin (redirect/get)
               (main-page req #f)))))

(define (handle-all-links req)
  (value->json (all-links)))

(define (handle-add-link req)
  (let ([url (param-string req 'url)]
        [caption (param-string req 'caption)]
        [desc (param-string req 'desc)])
    (if (and url desc)
        (let ([l (create-link #:url url #:desc desc #:caption (and caption ""))])
          (value->json l))
        (value->json #f))))

(define (all-links)
  (for/list (((k v) links)) v))
    
;; CHANGE THIS TO INCLUDE YOUR AD IN THE PAGE
(define (pick-ad)
  (include-template "ads/apply-ad.html"))
